#include "caesar.h"

#define BUFFER_SIZE 128

/*
 This function read the file block by block until the end of it
 */
int readFile(FILE *f, int *size, char *buffer){
    int index = 0;
    char c;
    int end = 0;

    do{
        c = fgetc(f);
        buffer[index] = c;
        index++;
        *size += 1;

        // End of the file, we stop it
        if (c == EOF){
            buffer[index - 1] = '\0';
            index = BUFFER_SIZE;
            end = 1;
        }
    }while(index < BUFFER_SIZE);

    return end;
}
/*
	s -> path source
	d -> path destination
	key -> key of encryption
	This function encrypt data
*/
int cryptCaesar(const char *s, const char *filenameDst, const int key) {
	int i;
	char c = 0;
    FILE *f = NULL, *fDst = NULL;
    char data[BUFFER_SIZE];
    char dataEncrypted[BUFFER_SIZE];
    int size = 0, end = 0;

    // Open the file
    f = fopen(s, "r+");
    if (f == NULL) return -1;

    fDst = fopen(filenameDst, "w");
    if (fDst == NULL) {
        fclose(f);
        return -1;
    }

    // We are going to read the file block by block
    do {
        end = readFile(f, &size, data);

	    /********* Encryption **********/
        for (i = 0; i < size; i++){
            if (data[i] >= 'A' && data[i] <= 'Z'){
                c = data[i] - 'A';
                dataEncrypted[i] = ((c + key) % 26) + 'A';
            }
            else if (data[i] >= 'a' && data[i] <= 'z'){
                c = data[i] - 'a';
                dataEncrypted[i] = ((c + key) % 26) + 'A';
            }
            else dataEncrypted[i] = data[i];

            // We can write into the dest file
            if (dataEncrypted[i] != '\0')
                fputc(dataEncrypted[i], fDst);
        }
        size = 0; 

        memset(data, 0, BUFFER_SIZE);
        memset(dataEncrypted, 0, BUFFER_SIZE);
    }while(end != 1);

	/* Close files */
    fclose(f);
    fclose(fDst);
    return 0;
}
/*
	s -> path source
	d -> path destination
	key -> key of decryption
	This function decryption by Cesar
*/
int decryptCaesar(const char *s, const char *filenameDst, const int key) {
	int i;
    FILE *f = NULL, *fDst = NULL;
    char data[BUFFER_SIZE];
    char dataDecrypted[BUFFER_SIZE];
    int size = 0, end = 0;

    // Open the file
    f = fopen(s, "r+");
    if (f == NULL) return -1;

    fDst = fopen(filenameDst, "w");
    if (fDst == NULL) {
        fclose(f);
        return -1;
    }

    // We are going to read the file block by block
    do {
        end = readFile(f, &size, data);

	    /********* Decryption **********/
        for (i = 0; i < size; i++){
            int val = 0;
            //if (data[i] >= 'A' && data[i] <= 'Z'){
		    if(data[i] >= 65 && data[i] <= 90) {
                val = data[i] - key;
                if (val < 'A')
                    val = val + 26;
            }
            else val  = data[i];
            dataDecrypted[i] = val;
            // We can write into the dest file
            if (dataDecrypted[i] != '\0')
                fputc(dataDecrypted[i], fDst);
        }
        size = 0; 

        memset(data, 0, BUFFER_SIZE);
        memset(dataDecrypted, 0, BUFFER_SIZE);
    }while(end != 1);

	/* Close files */
    fclose(f);
    fclose(fDst);

	return 0;
}
